//=============================================================================
// Copyright (c) 2016-present Allan CORNET (Nelson)
//=============================================================================
// This file is part of the Nelson.
//=============================================================================
// LICENCE_BLOCK_BEGIN
// SPDX-License-Identifier: LGPL-3.0-or-later
// LICENCE_BLOCK_END
//=============================================================================
#pragma once
//=============================================================================
#include "RandomInterface.hpp"
#include "nlsRandom_exports.h"
#include <boost/container/vector.hpp>
#include <boost/random.hpp>
#include <boost/random/variate_generator.hpp>
#include <string>
//=============================================================================
namespace Nelson {
class NLSRANDOM_IMPEXP RandomLaggedFibonacci607 : public RandomInterface
{

private:
    void* generator = nullptr;
    uint32 seed = 0;
    boost::lagged_fibonacci607 rngLaggedFibonacci607;
    boost::variate_generator<boost::lagged_fibonacci607&, boost::uniform_real<>>*
        uniform_real_generator;
    boost::variate_generator<boost::lagged_fibonacci607&,
        boost::random::uniform_int_distribution<>>* uniform_int_generator;
    boost::variate_generator<boost::lagged_fibonacci607&, boost::normal_distribution<>>*
        normal_real_generator;

public:
    RandomLaggedFibonacci607();
    ~RandomLaggedFibonacci607() override;

    std::wstring
    getGeneratorName() override;

    void
    setSeed(uint32 _seed);
    uint32
    getSeed();

    double
    getValueAsDouble(RNG_DISTRIBUTION_TYPE _type = RNG_DISTRIBUTION_UNIFORM_REAL) override;
    single
    getValueAsSingle(RNG_DISTRIBUTION_TYPE _type = RNG_DISTRIBUTION_UNIFORM_REAL) override;

    void
    getValuesAsDouble(double* ar, indexType nbElements, indexType lastDim,
        RNG_DISTRIBUTION_TYPE _type = RNG_DISTRIBUTION_UNIFORM_REAL) override;
    void
    getValuesAsSingle(single* ar, indexType nbElements, indexType lastDim,
        RNG_DISTRIBUTION_TYPE _type = RNG_DISTRIBUTION_UNIFORM_REAL) override;

    boost::container::vector<uint32>
    getState();
    void
    setState(const boost::container::vector<uint32>& _state);
    void
    setState(uint32* _state, size_t len);
    size_t
    getStateSize() override;

    void
    setMinMaxUniformIntDistribution(int _min, int _max) override;
    void
    getMinMaxUniformIntDistribution(int& _min, int& _max) override;
};
} // namespace Nelson
//=============================================================================
